import os

VERSION_INFO = {
    'major': 1,
    'minor': 0,
    'micro': 1,
}


def get_version(short=False):
    """Get a string with app version.

    """
    version = "%(major)i.%(minor)i" % VERSION_INFO
    # append micro version only if not short and micro != 0
    if not short and VERSION_INFO['micro']:
        version = version + (".%(micro)i" % VERSION_INFO)

    return version


def read_file(*path):
    """Read and return all contents of a text file

    """
    full_path = os.path.join(*path)
    file_obj = open(full_path)
    try:
        return file_obj.read()
    finally:
        file_obj.close()


__version__ = get_version()



import hashlib
import logging
import os
import shutil

from ConfigParser import ConfigParser
from ConfigParser import NoSectionError

from shrink import utils

OPEN_FILE_ERROR = u'Unable to open file %s'

LOG = logging.getLogger('shrink')


class ParserError(Exception):
    """Parser exception class

    """


class Parser(object):
    """Shrink ini file parser class

    """
    def __init__(self, cfg_file, args):
        self.cfg_file_dir = os.path.dirname(cfg_file)
        self.cfg_parser = self.get_config_parser(cfg_file)
        self.cfg_defaults = self.cfg_parser.defaults()
        self.args = self.update_default_arguments(args)
        self.sha1_hash = hashlib.sha1()
        # set hash file directory
        self.sha1_dir = self.get_sha1_dir()
        # when true a sha1 file is generated after minification
        self.sha1_updated = False

    def get_sha1_dir(self):
        """Get the full directory path for sha1 file

        """
        hash_dir = self.args.hash_dir
        if not hash_dir:
            hash_dir = (self.cfg_defaults.get('hash_dir') or '')
            # replace "here" variable with config file dir
            hash_dir = hash_dir % {'here': self.cfg_file_dir}

        # when no hash dir is given use config file directory
        return (hash_dir or self.cfg_file_dir)

    def get_config_parser(self, cfg_file):
        """Get a new ini file parser

        Parser is initialized with the given file information.

        """
        # init some default config parser variable values
        cfg_defaults = {'here': self.cfg_file_dir}
        # read config file options
        cfg_parser = ConfigParser(defaults=cfg_defaults)
        try:
            cfg_parser.read(cfg_file)
        except Exception:
            # print traceback onlu when debug flag is present
            if self.args.debug:
                LOG.exception(u'Config file is not valid')

            raise ParserError(u'Unable to parse config file %s' % cfg_file)

        return cfg_parser

    def update_default_arguments(self, args):
        """Update default values for missing command arguments using config
        file default values

        """
        for (name, value) in self.cfg_defaults.items():
            #skip non argument values
            if not name.startswith('arg.'):
                continue

            # remove "arg." prefix for name to get argument name
            arg_name = name.replace('arg.', '')
            # when argument is missing use ini file value
            if getattr(args, arg_name, None) is None:
                LOG.debug(u'Setting arg %s=%s from ini', arg_name, value)
                setattr(args, arg_name, value)

        return args

    def update_sha1_hash(self, file_name, chunk_size=4096):
        """Update SHA1 hash object with file info

        """
        try:
            file_obj = open(file_name, 'rb')
        except Exception:
            if self.args.debug:
                LOG.exception(OPEN_FILE_ERROR, file_name)

            LOG.error(u'File %s not included in hash', file_name)
            return

        # get file contents in small chunks and update global hash
        while True:
            chunk = file_obj.read(chunk_size)
            if not chunk:
                break

            # update global hash object with current chunk digest
            chunk_digest = hashlib.sha1(chunk).hexdigest()
            self.sha1_hash.update(chunk_digest)
            self.sha1_updated = True

        file_obj.close()

    def save_sha1_hash(self):
        """Save shrink.sha1 file in base directory

        """
        digest = self.sha1_hash.hexdigest()
        file_name = os.path.join(self.sha1_dir, 'shrink.sha1')
        try:
            file_obj = open(file_name, 'w')
            file_obj.write(digest)
            file_obj.close()
            LOG.info(u'Saved hash file %s', file_name)
        except Exception:
            if self.args.debug:
                LOG.exception(OPEN_FILE_ERROR, file_name)

            raise ParserError(u'Unable to save hash to %s' % file_name)

    def parse_section(self, section_name):
        """Parse a config section and minify files for that section

        """
        LOG.debug(u'Parsing section [%s]', section_name)
        # create a dictionary with config file section values
        values = dict(self.cfg_parser.items(section_name))

        # check if current section is a section for a group
        # NOTE: including groups inside groups can lead to recursion
        if 'group' in values:
            LOG.debug(u'Parsing section group [%s]', section_name)
            section_name_list = values['group'].strip('\n').split('\n')
            for name in section_name_list:
                try:
                    self.parse_section(name)
                except NoSectionError:
                    msg = (u'Invalid section name "%s" in section group [%s]'
                           % (name, section_name))

                    raise ParserError(msg)

            # after processing the group stop section processing
            return

        # check mandatory section option values
        for name in ('source_directory', 'destination_file', 'source_files'):
            if not values.get(name):
                msg = u'No %s value in section [%s]' % (name, section_name)

                raise ParserError(msg)

        # get section option values
        source_directory = values['source_directory']
        destination_file = values['destination_file']
        # by default source dir is used as destination dir
        destination_directory = values.get('destination_directory',
                                           source_directory)
        destination_file = utils.get_absolute_path(destination_directory,
                                                   destination_file)

        source_files = values['source_files'].strip('\n')
        source_file_list = [utils.get_absolute_path(source_directory,
                                                    file_name)
                            for file_name in source_files.split('\n')]

        # when a string with list of files is available
        # join them into one single file
        source_file_count = len(source_file_list)
        if source_file_count > 1:
            # source file name is now the temporary file
            # with contents of all files in group
            source_file = utils.join_files(source_file_list)
            if not source_file:
                msg = (u'Unable to generate file %s for section [%s]'
                       % (destination_file, section_name))
                LOG.error(msg)

                return
        # when no files are joined just add directory to source file name
        elif source_file_list:
            source_file = source_file_list[0]
            source_file = utils.get_absolute_path(source_directory,
                                                  source_file)
        else:
            msg = u'Section [%s] dont have source file(s)' % section_name
            LOG.error(msg)

            return

        # check if source file has to be compressed (default is true)
        skip_compress = (values.get('compress', 'true') == 'false')
        if not skip_compress:
            # init keyword arguments for compress function
            kw = {}
            if self.args.java_bin:
                kw['java_bin'] = self.args.java_bin
            if self.args.yui_jar:
                kw['yui_jar'] = self.args.yui_jar

            # compress file and check if return code is not zero
            # when return code is not zero it means yuicompressor failed to run
            (return_code, out_str) = utils.compress(source_file,
                                                    destination_file,
                                                    **kw)
        else:
            LOG.info(u'Generating file %s', destination_file)
            shutil.copy(source_file, destination_file)
            return_code = 0

        if return_code:
            LOG.error(u'Unable to generate %s', destination_file)
            # include yuicompressor error output
            if self.args.verbose:
                LOG.error(out_str)
        elif values.get('hash') or self.args.hash:
            # update sha1 hash object when file has to be included in hash
            self.update_sha1_hash(destination_file)

        # if files were joined into a single file remove
        #source file because is a temporary file
        if source_file_count > 1:
            os.remove(source_file)
            LOG.debug(u'Deleted temporary file %s', source_file)

    def get_section_names(self):
        """Get a list with all config file sections

        """
        # get section names sorted alphabetically
        section_name_list = self.cfg_parser.sections()
        section_name_list.sort()

        return section_name_list

    def list_all_sections(self):
        """List all sections in a config file

        """
        cfg_parser = self.cfg_parser
        section_name_list = self.get_section_names()
        # process each section
        for section_name in section_name_list:
            line = section_name
            is_section_group = cfg_parser.has_option(section_name, 'group')
            # when section is a group list sections inside that group
            if is_section_group:
                sub_sections = cfg_parser.get(section_name, 'group')
                sub_sections = sub_sections.strip('\n')
                sub_sections = sub_sections.replace('\n', u', ')

                line = u'%s [%s]' % (line, sub_sections)

            yield line

    def parse(self):
        """Parse sections in a config file for the given arguments

        """
        section_name_list = self.get_section_names()
        #section name "all" means all sections are parsed
        parse_all = ('all' in self.args.sections)
        if parse_all:
            self.args.sections = section_name_list

        # process each section
        for section_name in section_name_list:
            is_section_group = self.cfg_parser.has_option(section_name,
                                                          'group')
            # if section name is not available as argument skip it, and also
            # skip section groups when all is available as script parameter
            skip_section = ((section_name not in self.args.sections)
                            or (parse_all and is_section_group))
            if not skip_section:
                self.parse_section(section_name)

        # finally generate the sha1 hash file in base directory
        if self.sha1_updated:
            self.save_sha1_hash()



import logging
import os
import shutil
import tempfile

from subprocess import PIPE
from subprocess import Popen

JAVA_BIN = 'java'
YUI_JAR = 'yuicompressor.jar'

LOG = logging.getLogger('shrink')


def init_logging(options=None):
    """Init script logging

    """
    kw = {
        'format': '%(levelname)-5.5s %(message)s',
        'level': logging.INFO,
    }
    if options:
        if options.quiet:
            kw['level'] = logging.ERROR
        elif options.debug:
            kw['level'] = logging.DEBUG

    logging.basicConfig(**kw)


def compress(in_file_name, out_file_name, java_bin=None, yui_jar=None):
    """Run yuicompressor script for a file

    Runs compressor script and store result in a tuple with compressor
    script return code and output.
    Output is the error string when return code is not zero.

    Return a tuple with (result_code, output_str).

    """
    # set some sane default values
    if not java_bin:
        java_bin = JAVA_BIN
    if not yui_jar:
        yui_jar = YUI_JAR

    LOG.info(u'Generating file %s', out_file_name)
    #get compression type from file extension (css, js)
    file_type = os.path.splitext(out_file_name)[1]
    file_type = file_type.lstrip('.')

    cmd = [java_bin, '-jar', yui_jar, '-o', out_file_name,
           '--type', file_type, in_file_name]
    LOG.debug(u'Executing: %s', u' '.join(cmd))

    #create a process to run compressor and
    #dont display command output to console
    popen = Popen(cmd, stdout=PIPE, stderr=PIPE)
    return_code = popen.wait()

    #after process finishes get command output
    if return_code:
        output_str = popen.stderr.read()
    else:
        output_str = popen.stdout.read()

    return (return_code, output_str)


def get_absolute_path(directory, file_name):
    """Get absolute path for a file

    """

    relative_path = os.path.join(directory, file_name)

    return os.path.abspath(relative_path)


def join_files(file_name_list):
    """Join a list of files into a single temporary file

    Each file is treated as text file.

    Return a string with temporary file name and path.

    """
    failed = False
    #create a temporary file to concatenate file contents
    (tmp_file_fd, tmp_file_name) = tempfile.mkstemp(text=True)
    LOG.debug(u'Created temporary file %s for join', tmp_file_name)

    #use given file descriptor to open file
    tmp_file = os.fdopen(tmp_file_fd, 'w')

    #concatenate each file in list into the temporary file
    for file_name in file_name_list:
        if not os.path.isfile(file_name):
            LOG.error(u'Invalid file %s for concatenation', file_name)
            failed = True
            break

        src_file = None
        try:
            LOG.debug(u'Adding %s to temporary file', file_name)
            src_file = open(file_name, 'r')
            #copy file contents into temp file
            shutil.copyfileobj(src_file, tmp_file)
        except Exception:
            LOG.exception(u'Unable to concatenate file %s', file_name)
            failed = True
            #close current opened file
            if src_file:
                src_file.close()

            break

        #always close opened files when no exception are raised
        if src_file:
            src_file.close()

    tmp_file.close()
    if failed:
        #remove temp file before exit
        os.remove(tmp_file_name)
        LOG.debug(u'Deleted temporary file %s', tmp_file_name)

        return False

    return tmp_file_name


def get_data_dir():
    """Get absolute path for data directory

    """
    return os.path.join(os.path.dirname(__file__), 'data')


def get_data_file(file_name):
    """Get path for a file in data directory

    """
    data_dir = get_data_dir()

    return os.path.join(data_dir, file_name)



import os

from setuptools import find_packages
from setuptools import setup

from shrink import __version__
from shrink import read_file


CURRENT_DIR = os.path.dirname(__file__)
README = read_file(CURRENT_DIR, "README.rst")
CHANGELOG = read_file(CURRENT_DIR, "CHANGELOG.rst")
LONG_DESCRIPTION = u"{0}\n\n{1}".format(README, CHANGELOG)
AUTHORS = (
    ("Jer\xc3\xb3nimo Jos\xc3\xa9 Albi", "albi@wienfluss.net"),
)


setup(
    name="shrink",
    version=__version__,
    author=", ".join([a[0] for a in AUTHORS]),
    author_email=", ".join([a[1] for a in AUTHORS]),
    description="Tool for minification of css and javascript files",
    long_description=LONG_DESCRIPTION,
    url="https://bitbucket.org/jeronimoalbi/shrink",
    keywords="minify javascript css yuicompressor",
    license="BSD License",
    platforms=["OS Independent"],
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Utilities",
    ],
    entry_points={
        'console_scripts': [
            "shrink = shrink.command:run",
        ],
    },
)



import argparse
import logging
import os
import shutil
import sys

from shrink import get_version
from shrink import utils
from shrink.parser import Parser
from shrink.parser import ParserError
from shrink.utils import JAVA_BIN
from shrink.utils import YUI_JAR

DEFAULT_CFG = 'shrink.cfg'

LOG = logging.getLogger('shrink')


def get_argument_parser():
    """Get a new ArgumentParser for current shrink parameters

    """
    usage = 'shrink [options] (section [section ..] | all)'
    desc = 'Minify css and javascript files using an external minifier'
    formatter = argparse.RawTextHelpFormatter

    # initialize help text for all arguments
    help = {}
    help['cfg_file'] = 'shrink config file [default: %s]' % DEFAULT_CFG
    help['java_bin'] = 'java binary file [default: %s]' % JAVA_BIN
    help['yui_jar'] = 'yui compressor jar file [default: %s]' % YUI_JAR
    help['list_sections'] = 'list all config file sections'
    help['debug'] = 'enable debugging messages'
    help['verbose'] = 'show yuicompressor errors'
    help['quiet'] = 'only output errors to console'
    help['hash'] = 'use all files to create "shrink.sha1" hash file'
    help['hash_dir'] = ('directory to save "shrink.sha1" hash file\n'
                        '[default: config file directory]')
    help['example'] = 'copy example_shrink.cfg to current directory'
    help['version'] = 'output version information and exit'
    help['section'] = ('section name(s) to minify, or "all" '
                       'to minifiy all files')

    parser = argparse.ArgumentParser(usage=usage, description=desc,
                                     formatter_class=formatter)
    parser.add_argument('-f', '--cfg-file', dest='cfg_file',
                        help=help['cfg_file'])
    parser.add_argument('-j', '--java-bin', dest='java_bin',
                        help=help['java_bin'])
    parser.add_argument('-y', '--yui-jar', dest='yui_jar',
                        help=help['yui_jar'])
    parser.add_argument('-l', '--list-sections', action='store_true',
                        dest='list_sections', help=help['list_sections'])
    parser.add_argument('-d', '--debug', action='store_true', dest='debug',
                        help=help['debug'])
    parser.add_argument('-v', '--verbose', action='store_true',
                        dest='verbose', help=help['verbose'])
    parser.add_argument('-q', '--quiet', action='store_true', dest='quiet',
                        help=help['quiet'])
    parser.add_argument('--hash-all', action='store_true', dest='hash',
                        help=help['hash'])
    parser.add_argument('--hash-dir', dest='hash_dir', help=help['hash_dir'])
    parser.add_argument('--example-cfg', dest='example', action='store_true',
                        help=help['example'])
    parser.add_argument('--version', dest='version', action='store_true',
                        help=help['version'])
    parser.add_argument('sections', nargs='*', metavar='section',
                        help=help['section'])

    return parser


def print_config_sections(parser):
    """Print all sections in a config file for a given parser

    """
    for line in parser.list_all_sections():
        print line


def run():
    """Main entry point for shrink command

    """
    # parse shrink command arguments
    arg_parser = get_argument_parser()
    args = arg_parser.parse_args()

    # init config file absolute path
    cfg_file = (args.cfg_file or DEFAULT_CFG)
    cfg_file = os.path.abspath(cfg_file)

    if args.version:
        print 'Shrink %s' % get_version()
        sys.exit()
    elif args.example:
        # when --example-cfg is present copy example to
        # current folder and exit
        file_name = 'example_shrink.cfg'
        if os.path.isfile(file_name):
            print ('File %s not copied because it already exists in '
                   'current folder' % file_name)
            sys.exit(1)

        file_path = utils.get_data_file(file_name)
        shutil.copy(file_path, file_name)
        print 'Copied %s to current folder' % file_name
        sys.exit()
    elif not (args.sections or args.list_sections):
        #when no section name(s) is given or no section
        #listing is being made print command help and exit
        arg_parser.print_help()
        sys.exit()

    cfg_file_exists = os.path.isfile(cfg_file)
    if args.cfg_file and not cfg_file_exists:
        # when config file argument exist but
        # file does not inform user and exit
        print (u'File %s does not exist' % cfg_file)
        sys.exit(1)

    try:
        parser = Parser(cfg_file, args)
        utils.init_logging(options=parser.args)
        # print message when using default config file
        if not args.cfg_file:
            LOG.info(u'Using config file %s', cfg_file)

        if args.list_sections:
            # list config file sections (--list-sections)
            print_config_sections(parser)
        else:
            # by default parse config file and minify
            parser.parse()
    except ParserError, err:
        LOG.error(err)
        sys.exit(1)

    sys.exit()
